
# q-expansion of newform 441.8.a.l, downloaded from the LMFDB on 13 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 441
weight = 8
poly_data = [-216, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1, 1], [0, 0], [160, 10], [0, 0], [-1480, 116], [-1708, 882], [-906, 324], [-16834, 858], [3312, -48], [-15010, 9380], [197172, -2508], [170106, 27180], [379190, -23716], [-237424, -5628], [-563004, 36900], [-1432014, 5184], [53274, 53622], [403544, 57558], [-189788, 50928], [3684672, 130872], [-2054658, -191928], [-3342760, 277080], [5895834, -132594], [4704538, 363184], [-5428710, 94668], [6608480, -438646], [1313236, 1137084], [-11725636, 329960], [794538, 831204], [-7527046, -1214332], [-21196656, -752136], [10555474, 2200726], [-15611970, 553368], [-7644126, 2150022], [7653738, -500856], [219320, 529224], [9256628, -3241758], [-41093816, -2187564], [9073540, 3447052], [25643204, -2843194], [79478788, 1861864], [61925948, 574770], [30085784, 2797208], [23967758, -1348092], [-11628598, -3328696], [18684732, -5224284], [22747972, 8164800], [61008248, -13318200], [153680370, -7867578], [36754516, -12689862], [-244611786, 3469488], [117684264, 12499872], [81355354, 8756508], [-210596834, 3295642], [139931378, -8320624], [357544352, -13679368], [-302521404, 6602598], [-9567088, -417168], [-131634626, -9998496], [121291798, -17316320], [-412883818, 13962282], [-223314952, -19215742], [-323060962, -29110998], [411124872, -16524240], [16023610, 12609636], [168057954, -14654520], [118149296, -23630748], [95194318, -53913804], [254580400, 48567628], [-545766676, 18522], [-136739438, -15930080], [-123674648, -73248104], [-668762280, -9522312], [198383030, 39546744], [-816213480, -30008580], [-177381324, -9975228], [-1593161034, 43102308], [-164253060, 76689498], [1082913042, 15070812], [403062754, -83148228], [20550390, 25770570], [429293342, -88322640], [1025433672, -24421008], [450698738, -170805180], [-2642950344, -31359072], [-86434956, 136591008], [-2111286354, 17936688], [-2161215386, -89015676], [-454877668, -166598530], [-1741253456, -2108064], [491011002, 97332798], [-946245940, -97844500], [30063232, -92997552], [265483636, -175300496], [2998423588, -113441616], [-2081079840, -48840456], [2702567636, 68053478], [-1302666858, 316337100], [-1818841282, -101857710], [-979947970, -231778296], [-42114560, -149427348], [-4615683838, -65395816], [-4370761202, 22512898], [1902609962, -214381684], [2265235304, 53779596], [-972110626, -89332560], [-905351930, -386060318], [4134563418, 194842728], [2098097880, -496394160], [-3015054714, 146922384], [4789732480, -71598432], [4474267066, -300935268], [2167479690, 20832588], [363315494, 429338922], [4042719536, -136110408], [-3081857166, -46961652], [9530497186, 79056894], [587802884, 308731964], [-2693644218, 114913212], [9100768416, -322902972], [-5897339032, -229930290], [4582365370, 339152184], [4865012676, 201836190], [2911675764, 543879648], [4666100398, 188917530], [8480952870, 244275276], [704873994, 759352572], [3479947260, 298921500], [-8305747772, 346737132], [9092238408, 696923910], [6471846216, -51074412], [-16395146280, -337499064], [-5004188456, 1082184816], [-13525838758, -197846292], [13013987454, -243855084], [-1438982174, -224433132], [2444967032, 1185178850], [17362277558, -29911758], [-14999961788, 296306038], [7381722954, 814610892], [-14600101922, 199283490], [415722994, 516401488], [9334434840, -99463536], [-9621184596, 1048727400], [-3935875640, 272986950], [17678749564, 681475732], [18829294372, 895072122], [20069197726, -181345820], [5313444130, -841851930], [-282150840, -1453112184], [-15183814094, -941545620], [-9258664758, -988245720], [-12355919812, -597743832], [24630547724, 706440164], [12133118084, -118579416], [-3351016272, 691021800], [11340389528, 429873744], [15634665206, -1574472652], [17774256990, 77548968], [15303859400, -884153926], [6025167856, 1035719476], [-22775571674, -114649136], [6329363728, 2054799936], [-928219398, -2318384730], [-2288835258, -2683933416], [-6852022068, -1340424300], [-28282310368, -846873360], [-47829969352, 10659822], [-32006971778, 1763337156], [-3501918484, -2500559338], [-24668773692, 2059313904], [-27377585004, -278050206], [20641400928, -931398600], [33377727722, -1249159440], [22234317556, -137370732], [-23617251070, -236589640], [-8251479836, 727866216], [56667869990, 909694916], [-30337897296, -412055112], [6791136920, -2202426810], [15213437672, -1967780592], [-29541913170, 223572594], [-11352729022, 2091094068], [-6667183846, 3108801104], [15053703312, -3203099448], [-27376385452, 2455441214], [45339071830, -1100982768], [-56259414418, -477089958], [-49792198814, 2481063528], [-31440904988, 3868186660], [-20157952450, -1402910064], [-39797906556, -3670940112], [-11558133028, 3820494600], [28912338636, 1591731102], [-3899061820, 6818894408], [-51969152574, -3421518168], [-21883560178, 4111858740], [47350871490, 3041327484], [28876701882, 4674227208], [663286056, -1517068368], [27246095818, 3505158304], [51769465132, -4140420228], [-16333939320, 2845019118], [25300432454, -1465029288], [51626418578, 3339720734], [-15617933320, 2688036122], [2427298140, 4857824220], [-47228380632, 4299720300], [34216644306, 432414948], [25227756746, 3416710974], [-12989938914, -5031676140], [-19479044592, 3678012618], [-25017393224, 3208495752], [-40904618966, 4145846134], [29651559368, -9556961776], [-8286093262, 9384478380], [-86976993520, -3693741888], [56724613054, 3876587836], [-27240113704, -5182258864], [-72996843306, 1267387548], [-49167928510, -253430556], [44787338792, -5385602544], [-26639266898, -2479929584], [-64075234504, 6305399232], [-52762395742, 3725315846], [52054383698, 3590437284], [101951428522, 3132854560], [30092201416, -6245215952], [-167830487936, 948336120], [-150893451960, 3538694916], [24222247066, 2027142372], [-135938459542, 1897390374], [23695857488, -11278403472], [-12029086954, -2607864976], [-39938906322, -11173330350], [13772842752, -607490112], [107093750282, -7186809444], [64489061050, -2089543784], [142070659092, -3775623264], [-24222697408, -13791824200], [-183405062256, 657515268], [82238261022, -541282878], [-111179520, -6307134024], [-80856054602, -5884965696], [170750697642, -6123484920], [72997681252, 2458384876], [-222145935088, 1338351504], [186458471714, 7519698566], [43918527040, 1006414020], [119293043952, -647486448], [-166282883170, 2723718312], [34463346882, -6222366000], [21572028944, -1161960040], [124731732902, -5646098304], [-19477944104, 1764537322], [18312508180, 5163847480], [160303067522, 984521940], [144161119770, 10520509998], [-85824057312, 8276229066], [-21049053038, -4966732116], [-277008492856, -998913024], [247317791992, 3399660124], [23655282976, 12956084118], [-164210641112, 8359238790], [111272797094, 10127437364], [-212082810470, 1190727990], [168015201618, -6346911816], [174785647834, -14596826384], [-154518105068, -1427278440], [-36068854930, -8948757604], [-158548177444, 10478478714], [158378313488, -5501310084], [-150628490002, -513733320], [10708255424, -9245829048], [33911058734, 12570946080], [-94560066992, 19220431080], [86247357680, -14898305092], [101510510286, 23127891048], [-111072039506, -2056766244], [-237414846838, 14443077470], [-343777415320, 3213055064], [49792827864, 25112513856], [-17740254532, 21246999956], [77914664408, -23730914130], [415205259022, 9294249234], [-261696101232, -3097118520], [-398283291998, 6552642960], [122057917770, 14606167128], [-75866465756, -23228368788], [-151388138958, 16885050336], [-74144265478, 20055878672], [-237306927350, -15188364098], [-488070093510, -4352042484], [5326337058, -24315482562], [57246531862, -8892187920], [-14636336732, 21490612054], [-402870764512, -632874000], [-310170071596, 4470811646], [-551997604978, 6883826666], [76951890274, 23031361710], [-22631599710, 21754642956], [-113147294490, 3834742668], [-184915397888, -16259601264], [-80116598536, 35498433068], [151333735776, -7045787148], [-78739081062, 26631394776], [-89979135648, -12384842676], [-76380282048, -5063188914], [534178779632, 6734550656], [2703474766, -38665963008], [246730499448, -12149899752], [6952267270, -38404955900], [-67980037194, 15106299384], [-88649124456, -21708029004], [41927505856, 3480308776], [-526427900954, -11002201632], [-36345811026, -23243839038], [-267930782976, 19039473408], [509005037030, -4461390024], [-17733959490, 28324670640], [289959482746, 19667047734], [-92367545726, 43698277944], [401041917528, 16214137338], [-166302513592, 28565727288], [-288177086886, 10959092988], [514520881778, -17697061980], [50983419136, 19024685868], [462686397454, -14204489022], [-122420136656, -9180654440], [231202043742, -32377285668], [116032910062, -9678925320], [-354763124922, -22692752148], [146010138988, 36985207644], [856274966502, 5261408490], [351776111956, -45350732088], [733684725970, 23032395022], [186010545674, 8434073172], [-113423758638, -37813373280], [-601026123530, -19866102144], [-107245805092, -8386641588], [-549289361450, -8270288856], [-557157245990, 39691918588], [-572459697852, 18249409374], [-547010537120, -22887623976], [236550171670, 16013588956], [88404864560, -18335021740], [-36995073028, -46858781118], [479295131608, 24567667116], [376350485448, -25422058872], [-547755255160, -39708411838], [-516856780222, -25862803410], [426679945002, -12705251472], [397578497258, 17635698080], [-28294850868, -17448367428], [-199888668554, 47828991048], [-833238301966, 8804881736], [166095140364, -22141706388], [244219919178, -51242081322], [8699619790, -58119477632], [-218336825184, -38467180416], [-36525348858, 4289488728], [141154503202, 65533916920], [-456558332016, 52585518360], [17805367084, 57814943992], [38378979838, -473237718], [228193977766, 24611153412], [-705329760604, 7277122790], [737785710112, 28714074000], [-155489292810, -75343704156], [-698218102956, -30665692872], [-101752089102, 35099180694], [-709608298608, 56021481552], [813122670330, 30777017880], [-679349046296, 9441812448], [-4149760322, 45297556296], [687577714450, -6752433770], [497107687928, -66539996320], [656586573478, -17175197160], [1007797119174, -20240424348], [616997642562, 51679798044], [-874288228012, -9865172602], [-851017947894, 33603933804], [267769051384, 17662148784], [-1342331010002, -14380372688], [580384619682, 49339264926], [1664532065816, -3178088128], [72136981976, 75082112472], [1084112649056, -9926768154], [-33372261364, 103088633472], [1030877578500, 37438418052], [-573866936354, 23613584148], [376573185456, 73498835322], [727704158220, -64332925800], [113141246842, 29005528374], [-177011559080, -34643322104], [182233230322, -27993785880], [-261574586620, 92289289860], [-182465170650, 72101056860], [427073504916, -48105252888], [1091000741478, 41135686260], [-981486085880, 54232082790], [-611764327922, -25918058480], [69056189488, -52003743744], [605495452638, -33938329884], [-396503223252, -27911625306], [-108326208556, 18862589148], [-123997687146, -112383207696], [-1006533772862, -82167633380], [173169830074, -54892285002], [615762784398, 42659081874], [1413034702150, -12623039544], [-379573746530, -73796870132], [161585095096, 66293658004], [514914641044, -75406004304], [884123135226, -12131306880], [-241661951364, 75361989342], [1184258410728, -89579802696], [-927020613404, -97480458084], [-954586862586, -76295679516], [-316682745408, -116601164040], [501384219942, 3385884156], [276659181128, -139073858178], [-60472552688, 1898273104], [486488827286, -64694335342], [66587159338, 54070289700], [-1578767260416, 79154856594], [752186965328, -27970904380], [-250574241402, -49699208496], [290684270250, 86762064366], [473407193172, -105960662484]]
